# frozen_string_literal: true

module Hermes
  module Tools
    # Tools for specs
    module Specs
      def get_typed_value(type, iteration = 0)
        type_array = case type
                     when 'String'
                       %w[alpha bravo charlie]
                     when 'Boolean'
                       [false, true, true]
                     when 'Hash'
                       [
                           {'alpha' => 'bravo'},
                           {'charlie' => 'delta'},
                           {'echo' => 'foxtrott'}
                       ]
                     when '[String]'
                       [%w[alpha bravo], %w[charlie delta], %w[echo foxtrott]]
                     when 'Array'
                       [%w[alpha bravo], %w[charlie delta], %w[echo foxtrott]]
                     when 'Array[Coord]'
                       [[0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]
                     when 'Date'
                       [
                           [2002, 2, 3, 5],
                           [2003, 2, 3, 8],
                           [2004, 2, 3, 6]
                       ].map {|arr| Date.new(*arr)}
                     when 'DateTime'
                       [
                           [2001, 2, 3, 4, 4, 4],
                           [2002, 2, 3, 4, 4, 6],
                           [2003, 2, 3, 4, 4, 6]
                       ].map {|arr| DateTime.new(*arr)}
                     when 'Integer'
                       [1, 2, 3]
                     when 'Float'
                       [1.1, 2.2, 3.3]
                     else
                       puts "ERROR: Unknow type #{type}"
                       [nil, nil, nil]
                     end
        type_array[iteration]
      end

      def nice_type(type)
        case type
        when 'String'
          nice_type = :string
        when 'Boolean'
          nice_type = :boolean
        when 'Hash'
          nice_type = :object
        when 'Array'
          nice_type = :array
        when 'Date'
          nice_type = :date
        when 'DateTime'
          nice_type = :time
        when 'Integer'
          nice_type = :integer
        when 'Float'
          nice_type = :float
        else
          puts 'ERROR: Unknow type'
        end
        nice_type
      end

      def jsonify_params(params)
        schema = {
            type: :object,
            properties: {
                data: {
                    type: :object,
                    properties: {
                        attributes: {
                            type: :object,
                            properties: {},
                            required: []
                        }
                    }
                    #required: []
                }
            }
            #required: []
        }
        params.each do |key, val|
          if val[:type] == :array
            schema[:properties][:data][:properties][:attributes][:properties][key] = {type: :array, items: { type: :string } }
          else
            schema[:properties][:data][:properties][:attributes][:properties][key] = {type: val[:type]}
          end
        end
        schema
      end
    end
  end
end

include Hermes::Tools::Specs
