require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  describe Api::Rex::RexController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:entity1) { create(:entity, siret:    'ABC00000000000', title: 'etp1') }
    let!(:entity1a) { create(:entity, siret:   'ABCA0000000000', title: 'etp11', parent_id: entity1.id) }
    let!(:entity1aA) { create(:entity, siret:  'ABCAA000000000', title: 'etp111', parent_id: entity1a.id) }
    let!(:entity1aAI) { create(:entity, siret: 'ABCAAA00000000', title: 'etp1111', parent_id: entity1aA.id) }

    let!(:entity2) { create(:entity, siret: 'BCA00000000000', title: 'etp2') }
    let!(:entreprise_user) { create(:user, auth_roles:%w[entreprise], current_auth_role: 'entreprise') }

    context 'with entreprise_user within entity readers group' do
      before :each do
        # TODO: change to cre_group once merged
        ((1..2).to_a + %w[1a 1aA 1aAI]).each do |i|
          entity = eval("entity#{i}")
          entity.set(root_id: entity.id, updated_at: DateTime.current) if entity.root_id.blank?
          supervisor_group = entity.supervisor_group
          supervisor_group.user_ids = supervisor_group.user_ids + [entreprise_user.id]
          supervisor_group.save

          authenticated_header(request, entreprise_user)
        end
      end

      describe 'GET root_elements' do
        before :each do
          get 'root_elements'
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          datas = json['data']
          expect(datas.length).to be > 0
          expect(datas.map { |elt| elt['id'] }).to include(entity1.id.to_s).and(include(entity2.id.to_s))
          (1..2).each do |i|
            entity = eval("entity#{i}")
            expect(datas.find { |elt| elt['id'] == entity.id.to_s }['attributes']['title']).to eq entity.title
          end
        end
      end

      describe 'GET show with id.blank?' do
        before :each do
          get 'show', id: ''
        end

        it 'returns status code 422' do
          expect(response).to have_http_status(422)
        end
      end

      describe 'GET show with id=entity1 & for_lessor=true' do
        before :each do
          get 'show', id: entity1.id.to_s, for_lessor: 'true'
        end

        it 'returns status code 422' do
          expect(response).to have_http_status(422)
        end
      end

      describe 'GET show with id=entity1' do
        before :each do
          get 'show', id: entity1.id.to_s
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid json' do
          data = json['data']
          ap data
          expect(data['from_element']['id']).to eq entity1.id.to_s
          expect(data['tree'].length).to be 1
        end
      end
    end
  end

  # describe Api::Rex::RexController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  #   TAGS_API_REX= 'API_REX_REX'
  #   include JwtAuthHelper
  #   extend HermesSwaggerJsonParams
  #   include SwaggerJsonAttributes

  #   let!(:user) { create(:user) }
  #   let!(:entreprise) { create(:user, auth_roles: %w[entreprise], current_auth_role: "entreprise") }
  #   let!(:bailleur) { create(:user, auth_roles: %w[bailleur], current_auth_role: "bailleur") }

  #   context 'as entreprise' do
  #     path '/api/rex/root_elements' do
  #       get 'Find root elements' do
  #         tags TAGS_API_REX
  #         produces 'application/json'
  #         consumes 'application/json'
  #         security [Bearer: {}]
  #         parameter(
  #           name: :for_lessor,
  #           in: :query,
  #           type: :string
  #         )

  #         response '200', 'root elements list' do
  #           let(:Authorization) {"Bearer #{token_for(entreprise)}"}
  #           let(:params) do
  #           end
  #           include_context 'with integration test'
  #           include_context 'valid jsonapi'
  #           include_context 'not containings errors'

  #           after do |_example|
  #             json = JSON.parse(response.body)

  #             expect(json['data']).to be_a Hash
  #             expect(json['data'].length).to be == 0
  #           end
  #         end
  #       end
  #     end
  #   end
  # end
end
